from ..styles import Style1, Style2,  Style3
from .valmanager import get_val, get_val_dict, set_val, reset_config
import os
from bs4 import BeautifulSoup
import requests
import re
import json
import webbrowser


#combine html
class Parser:
    def __init__(self):
        self.dark = ['#23272a', '#2C2F33', 'white', 'white', '#99aab5', '#99aab5']
        self.light = ['#f0f0f0', '#ffffff', '#333333', '#333', '#666', '#ccc']


    def parseAll(self, parameters : dict, optionals : dict, badges : dict) -> str:
        #parse theme
        self.style = optionals['style']

        if self.style == '1':
            _style = Style1()
            css, profile, id, badge, info, contect, end = _style()

        elif self.style == '2':
            _style = Style2()
            css, profile, id, badge, info, contect, end = _style()

        elif self.style == '3':
            _style = Style3()
            css, profile, id, badge, info, contect, end = _style()

        else:
            raise ValueError(f"Theme:{self.style} is not supported")

        #parse type
        self.type_parse(parameters['type'], parameters['profile_page_link'], parameters['profile_pic_link'], parameters['profile_name'], optionals['profile_id'])

        #parse options
        self.background_pic = optionals['background_pic_link']
        self.theme = optionals['theme']
        self.site_title = optionals['site_title']
        self.card_title = optionals['card_title']
        self.info = optionals['info']
        self.contect = optionals['contect']

        #parse badges
        if badges['badge0'] == "":
            self.badges = None

        else:
            self.badges = badges

        return self.combine(css, profile, id, badge, info, contect, end)

    def type_parse(self, type, link, profile_pic_link, profile_name, profile_id) -> None:
        if type == 'github':
            self.github_parse(link)

        elif type == 'youtube':
            self.youtube_parse(link)

        elif type == 'reddit':
            self.reddit_parse(link)

        elif type == 'leetcode':
            self.leetcode_parse(link)

        elif type == 'custom':
            self.custom_parse(profile_pic_link, profile_name, profile_id)

        else:
            raise ValueError(f'Type:{type} is not supported')


    def github_parse(self, link) -> None:
        soup = BeautifulSoup(requests.get(link).text, 'html.parser')
        self.profile_pic = soup.find("img").get("src")
        self.profile_name = soup.find("span", itemprop="name").text.strip()
        self.profile_id = soup.find("span", itemprop="additionalName").text.strip()

        if self.profile_pic is None or self.profile_name is None:
            raise ValueError(f'profilecard Error : Invalid link')


    def youtube_parse(self, link) -> None:
        soup = BeautifulSoup(requests.get(link).content, "html.parser")
        data = re.search(r"var ytInitialData = ({.*?});</script>", str(soup)).group(1)
        json_data = json.loads(data)
        self.profile_pic = json_data["header"]["c4TabbedHeaderRenderer"]["avatar"]["thumbnails"][0]["url"]
        self.profile_name = json_data["header"]["c4TabbedHeaderRenderer"]["title"]
        self.profile_id = None


    def reddit_parse(self, link) -> None:
        soup = BeautifulSoup(requests.get(link).text, 'html.parser')
        self.profile_pic = soup.find("img").get("src")
        self.profile_name = soup.find("h1").text.strip()
        self.profile_id = soup.find("p").text.strip()


    def leetcode_parse(self, link) -> None:
        temp = link.split('/')
        if temp[-1]:
            account = temp[-1]
        else:
            account = temp[-2]

        original_string = '{ matchedUser(username: "account") { profile { realName userAvatar } } }'
        query = original_string.replace('account', account)
        data = {
            "query": query,
        }
        res = requests.post(url='https://leetcode.com/graphql/', json=data)
        profile = res.json()["data"]['matchedUser']['profile']
        self.profile_pic = profile['userAvatar']
        self.profile_name = profile['realName']
        self.profile_id = None


    def custom_parse(self, profile_pic_link, profile_name, profile_id) -> None:
        self.profile_pic = profile_pic_link
        self.profile_name = profile_name
        if profile_id:
            self.profile_id = profile_id

        else:
            self.profile_id = None


    def combine(self, css, profile, id, badge, info, contect, end) -> str:
        # format css
        if self.theme == 'dark':
            css_area = css.format(
                site_title=self.site_title, t0=self.dark[0], t1=self.dark[1],
                t2=self.dark[2], t3=self.dark[3], t4=self.dark[4], t5=self.dark[5],
                background_pic_link=self.background_pic, profile_pic_link=self.profile_pic)

        elif self.theme == 'light':
            css_area = css.format(
                site_title=self.site_title, t0=self.light[0], t1=self.light[1],
                t2=self.light[2], t3=self.light[3], t4=self.light[4], t5=self.light[5],
                background_pic_link=self.background_pic, profile_pic_link=self.profile_pic)

        else:
            raise ValueError(f'Theme:{self.theme} is not supported')

        # format profile
        if self.profile_name == "":
            raise ValueError(f'ProfileCard Error : profile name is required')

        if self.background_pic:
            profile_area = profile.format(bg="bg", profile_name=self.profile_name, card_title=self.card_title)

        else:
            profile_area = profile.format(bg="", profile_name=self.profile_name, card_title=self.card_title)

        #format id
        if self.profile_id:
            id_area = id.format(profile_id=self.profile_id)
        else:
            id_area = "<br>"

        #format badge
        if self.badges is None:
            badge_area = ""
        else:
            badge_area = ""
            for key, value in self.badges.items():
                value = value.split(',')
                if value[5] == "":
                    badge_area += badge.format(badge_icon=value[0], badge_style=value[1], badge_color=value[2],
                                               badge_bg_color=value[3], badge_text=value[4],
                                               badge_link="")
                else:
                    badge_area += badge.format(badge_icon=value[0], badge_style=value[1], badge_color=value[2],
                                               badge_bg_color=value[3], badge_text=value[4],
                                               badge_link=f' href="{value[5]}" target = "_blank"')

        #format info
        if self.info:
            info_area = info.format(info=self.info)
        else:
            info_area = ""

        #format contect
        if self.contect:
            contect_area = contect.format(contect_text=self.contect)
        else:
            contect_area = ""

        #end
        end_area = end

        #combine all
        return "\n".join([css_area, profile_area, id_area, badge_area, info_area, contect_area, end_area])


#pack objects into html
class Pack(Parser):
    def __init__(self):
        self._parameters = dict(get_val_dict("Parameters"))
        self._optionals = dict(get_val_dict("Optional"))
        self._badges = dict(get_val_dict("Badge"))

        super().__init__()
        self.text = self.parseAll(self._parameters, self._optionals,  self._badges)

        if self.text:
            self.save()
        else:
            raise ValueError("profilecard Error : Pack failed")


    def save(self):
        current_folder = os.path.dirname(os.path.abspath(__file__))
        file_name = "profile.html"
        self.file_name = os.path.join(current_folder, file_name)

        with open(self.file_name, 'w') as f:
            f.write(self.text)


    def show(self):
        run = get_val(type="Run", name="first")
        if run == "false":
            webbrowser.open(self.file_name)
        elif run == "true":
            webbrowser.open(self.file_name)
            set_val(type="Run", name="first", val="stop")
        elif run == "stop":
            pass


    def reset(self):
        reset_config()


def show(*, first: bool = False):
    """
    This function is use to show profile card without packing
    """
    if get_val(type="Run", name="first") == "stop" and first == True:
        pass
    else:
        set_val(type="Run", name="first", val=str(first).lower())

    current_folder = os.path.dirname(os.path.abspath(__file__))
    file_name = "profile.html"
    file_name = os.path.join(current_folder, file_name)

    if os.path.exists(file_name) is not True:
        raise ValueError(f"profilecard Error : You need to pack first")

    run = get_val(type="Run", name="first")
    if run == "false":
        webbrowser.open(file_name)
    elif run == "true":
        webbrowser.open(file_name)
        set_val(type="Run", name="first", val="stop")
    elif run == "stop":
        pass





__version__ = "1.0.8"

from .simple import Pack, show
from .simple import set_val, get_val, reset_config as rc


class Pf:
    """
    This class is use to setup a personal profile card
    """
    def __init__(self, *, first : bool = False):
        self.style = "1"
        self.type = None
        self.link = None
        self.pic = None
        self.name = None
        self.stitle = 'About Author'
        self.ctitle = 'About Me'
        self.theme = 'dark'
        self.bg = None
        self.info = None
        self.contect = None
        self.badge = None

        self.badge_counter = 0

        if get_val(type="Run", name="first") == "stop" and first == True:
            pass
        else:
            set_val(type="Run", name="first", val=str(first).lower())


    def set_style(self, style : str = "1"):
        """
        This function is use to set the style of profile card
        ('1' , '2' , '3')
        (Default : '1')
        """
        set_val(type='Optional', name="style", val=style)

        self.style = style

        return f"Style:{self.style}"


    def set_type(self, *, type : str, link : str = ""):
        """
        This function is use to set the type of profile card
        type : support -> ('github' , 'youtube' , 'reddit' , 'custom')
        link : ('LINK OF PROFILE PAGE')
        """
        if type == 'custom':
            set_val(type='Parameters', name="type", val=type)

        elif type in ['github', 'youtube', 'reddit', 'leetcode']:
            set_val(type='Parameters', name="type", val=type)
            set_val(type='Parameters', name="profile_page_link", val=link)

        else:
            raise ValueError(f'ProfileCard Error : Invalid type "{type}"')

        self.type = type
        self.link = link

        return f"Type:{self.type} Link:{self.link}"


    def set_pic(self, link : str):
        """
        This function is use to set the profile picture for custom site
        (Default : 'https://cdn.pixabay.com/photo/2015/10/05/22/37/blank-profile-picture-973460_960_720.png')
        """
        set_val(type='Parameters', name="profile_pic_link", val=link)

        self.pic = link

        return f"Pic:{self.pic}"


    def set_name(self, name : str):
        """
        This function is use to set the name for custom site
        """
        set_val(type='Parameters', name="profile_name", val=name)

        self.name = name

        return f"Name:{self.name}"


    def set_site_title(self, title : str):
        """
        This function is use to set the title
        (Default : 'About Author')
        """
        set_val(type='Optional', name="site_title", val=title)

        self.stitle = title

        return f"Site Title:{self.stitle}"


    def set_card_title(self, title : str):
        """
        This function is use to set the title
        (Default : 'About Me')
        """
        set_val(type='Optional', name="card_title", val=title)

        self.ctitle = title

        return f"Card Title:{self.ctitle}"


    def set_theme(self, theme : str):
        """
        This function is use to set the theme
        (light , dark)
        """
        set_val(type='Optional', name="theme", val=theme)

        self.theme = theme

        return f"Theme:{self.theme}"


    def set_background_pic(self, link : str):
        """
        This function is use to set the background picture
        (Default : None)
        """
        set_val(type='Optional', name="background_pic_link", val=link)

        self.bg = link

        return f"Background Pic:{self.bg}"


    def add_id(self, id : str = ""):
        """
        This function is use to add the id text
        (Default : "")
        """
        set_val(type='Optional', name="profile_id", val=id)

        self.id = id

        return f"ID:{self.id}"


    def add_info(self, info : str):
        """
        This function is use to add the info text
        (Default : None)
        """
        set_val(type='Optional', name="info", val=info)

        self.info = info

        return f"Info:{self.info}"


    def add_contect(self, contect : str):
        """
        This function is use to add the contect text
        (Default : None)
        """
        set_val(type='Optional', name="contect", val=contect)

        self.contect = contect

        return f"Contect:{self.contect}"


    def add_badge(self, *, icon : str, style : str = "for-the-badge", color : str = "white", bg_color : str = "black", text : str = "_", link : str = ""):
        """
        This function is use to add the badge
        icon(Required) : ('github' , 'youtube' , 'facebook' , 'twitter' , 'instagram' , 'reddit' , 'gmail' and more...)(More badge icons : https://simpleicons.org/)
        style : ('flat' , 'flat-square' , 'plastic' , 'for-the-badge' , 'social')(Default : 'for-the-badge')
        color : (Default : 'white')
        bg_color : (Default : 'black')
        text : (Default : None)
        link : (Default : None)
        """

        if icon:
            set_val(type='Badge', name=f"badge{self.badge_counter}",
                    val=f"{icon},{style},{color},{bg_color},{text},{link}")

            self.badge_counter += 1
            self.badge = f"Badge:{icon},{style},{color},{bg_color},{text},{link}"

            return f"Badge:{self.badge}"

        else:
            raise ValueError("ProfileCard Error : badge icon are required")


    def pack(self):
        """
        This function is use to pack the profile card
        """
        self.pack = Pack()


    def show(self):
        """
        This function is use to show the profile card
        """
        self.pack.show()


    def reset(self):
        """
        This function is use to reset the config file
        """
        self.pack.reset()


def reset_profilecard_config():
    """
    This function is use to reset the config file
    """
    rc()


def show_without_repacking(*, first : bool = False):
    """
    This function is use to show the profile card without repacking
    """
    show(first=first)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='profilecard',
    version='1.0.8',
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    package_data={'': ['*.ini']},
    install_requires=[
        'beautifulsoup4',
        'requests',
    ],
)



class Style3:
    def __init__(self):
        self.css = '''
                <!DOCTYPE html>
                <head>
                    <title>{site_title}</title>
                    <style>
                        body {{
                            font-family: Arial, sans-serif;
                            display: flex;
                            justify-content: center;
                            align-items: center;
                            height: 100vh;
                            margin: 0;
                            background-color: {t0};
                        }}
                        .profile-card {{
                            display: flex;
                            width: 600px;
                            background-color: {t1};
                            border-radius: 10px;
                            box-shadow: 0 2px 10px rgba(0,0,0,0.2);
                            color: {t2};
                            padding: 20px;
                            text-align: center;
                        }}
                        .profile-card .profile-image {{
                            width: 200px;
                            height: 200px;
                            border-radius: 50%;
                            margin-right: 20px;
                            background-image: url('{profile_pic_link}');
                            background-position: center center;
                            background-size: cover;
                            background-repeat: no-repeat;
                            overflow: hidden;
                        }}

                        .profile-card .profile-info {{
                            display: flex;
                            flex-direction: column;
                            justify-content: center;
                            width: 100%;
                            text-align: left;
                        }}
                        .profile-card h2 {{
                            margin: 0;
                            padding: 0;
                            font-size: 24px;
                            color: {t3};
                        }}
                        .profile-card .id {{
                            color: {t4};
                        }}
                        .profile-card .line {{
                            margin-top: 15px;
                            border-top: 1px solid {t5};
                            padding-top: 10px;
                        }}
                        .profile-card img.badge {{
                            border-radius: 0;
                            width: auto;
                            height: auto;
                        }}
                        .bg {{
                            background-image: url('{background_pic_link}');
                            background-position: center center;
                            background-size: contain;
                            background-repeat: no-repeat;
                        }}
                    </style>
                    '''

        self.profile = '''
                </head>
                <body class="bg">
                    <div class="profile-card">
                        <div class="profile-image"></div>
                        <div class="profile-info">
                            <h1>{card_title}</h1>
                            <h2>{profile_name}</h2>
        '''

        self.id = '''
                <p class="id">{profile_id}</p>
                '''

        self.badge = '''
                <a{badge_link}><img class="badge" alt="Static Badge" src="https://img.shields.io/badge/{badge_text}-_?style={badge_style}&logo={badge_icon}&logoColor={badge_color}&color={badge_bg_color}"></a>
                '''

        self.info = '''
                <div class="line">
                    <h2>Info</h2><br>
                    <span style="display: block;">{info}</span><br>
                </div>
        '''

        self.contect = '''
                <div class="line">
                    <h2>Contact Me</h2>
                    <p>{contect_text}</p>
                </div>
        '''

        self.end = '''
                    </div>
                </body>
                </html>
        '''

    def __call__(self):
        return self.css, self.profile, self.id, self.badge, self.info, self.contect, self.end



class Style2:
    def __init__(self):
        self.css = '''
                <!DOCTYPE html>
                <head>
                    <title>{site_title}</title>
                    <style>
                        body {{
                            font-family: 'Helvetica Neue', sans-serif;
                            display: flex;
                            justify-content: center;
                            align-items: center;
                            height: 100vh;
                            margin: 0;
                            background-image: url('{background_pic_link}');
                            background-position: center center;
                            background-size: cover;
                            background-repeat: no-repeat;
                            background-color: {t0};
                        }}
                        .profile-card {{
                            display: flex;
                            width: 600px;
                            background-color: {t1};
                            border-radius: 15px;
                            box-shadow: 0 5px 20px rgba(0,0,0,0.25);
                            color: {t2};
                            overflow: hidden;
                        }}
                        .profile-image {{
                            background-image: url('{profile_pic_link}');
                            background-position: center center;
                            background-size: cover;
                            background-repeat: no-repeat;
                            width: 40%; /* Adjusted for aspect ratio */
                            height: auto;
                            min-height: 100%; /* Ensure it covers vertically */
                            box-sizing: border-box;
                            display: flex;
                            align-items: center; /* Center content vertically */
                            justify-content: center; /* Center content horizontally */
                            border-right: 2px solid {t5}; /* Stylish right border */
                        }}
                        .profile-info {{
                            width: 60%;
                            padding: 20px;
                            text-align: left;
                        }}
                        .profile-info h1, .profile-info h2 {{
                            margin-top: 0;
                        }}
                        .profile-info h1 {{
                            color: {t3};
                            padding-bottom: 10px;
                            border-bottom: 2px solid {t5};
                        }}
                        .profile-info .id {{
                            color: {t4};
                        }}
                        .profile-info .line {{
                            margin-top: 10px;
                            border-top: 2px solid {t5};
                            padding-top: 10px;
                        }}
                        .profile-info img.badge {{
                            width: auto;
                            height: auto;
                        }}
                    </style>
                    '''

        self.profile = '''
                </head>
                <body class="bg">
                    <div class="profile-card">
                        <div class="profile-image"></div>
                        <div class="profile-info">
                            <h1>{card_title}</h1>
                            <h2>{profile_name}</h2>
        '''

        self.part_id = '''
                            <p class="id">{profile_id}</p>
        '''

        self.badge = '''
                            <a{badge_link}><img class="badge" alt="Badge" src="https://img.shields.io/badge/{badge_text}-_?style={badge_style}&logo={badge_icon}&logoColor={badge_color}&color={badge_bg_color}"></a>
        '''

        self.info = '''
                            <div class="line">
                                <h2>Info</h2>
                                <span style="display: block;">{info}</span>
                            </div>
        '''

        self.contect = '''
                            <div class="line">
                                <h2>Contact Me</h2>
                                <p>{contect_text}</p>
                            </div>
        '''

        self.end = '''
                        </div>
                    </div>
                </body>
                </html>
        '''

    def __call__(self):
        return self.css, self.profile, self.part_id, self.badge, self.info, self.contect, self.end



import configparser
import os

current_folder = os.path.dirname(os.path.abspath(__file__))
file_name = "config.ini"
file_name = os.path.join(current_folder, file_name)

#use to get config value
def get_val(*, type : str, name : str):
    config = configparser.ConfigParser()
    config.read(file_name)

    return config[type][name]

#use to set config value
def set_val(*, type : str, name : str, val : str):
    config = configparser.ConfigParser()
    config.read(file_name)
    config.set(type, name, val)
    with open(file_name, 'w') as configfile:
        config.write(configfile)

#use to get config dict
def get_val_dict(type : str):
    config = configparser.ConfigParser()
    config.read(file_name)
    return config[type]


def reset_config():
    config = configparser.ConfigParser()

    config['Run'] = {
        'first': 'false'
    }
    config['Parameters'] = {
        'type': '',
        'profile_page_link': '',
        'profile_pic_link': 'https://cdn.pixabay.com/photo/2015/10/05/22/37/blank-profile-picture-973460_960_720.png',
        'profile_name': ''
    }
    config['Optional'] = {
        'style': "1",
        'theme': 'dark',
        'site_title': 'About Author',
        'card_title': 'About Me',
        'background_pic_link': '',
        'profile_id': '',
        'info': '',
        'contect': ''
    }
    config['Badge'] = {
        'badge0': ''
    }

    with open(file_name, 'w') as configfile:
        config.write(configfile)

    print("Configuration reset to default values.")


if __name__ == '__main__':
    a = "1"
    set_val(type="Optional", name="style", val=a)